@@ -23,12 +23,10 @@
  ***************************************************************************/
 #include "tool_setup.h"
 
-#define ENABLE_CURLX_PRINTF
-/* use our own printf() functions */
-#include "curlx.h"
-
 #include "tool_cfgable.h"
 #include "tool_msgs.h"
+#include "tool_cb_prg.h"
+#include "terminal.h"
 
 #include "memdebug.h" /* keep this as LAST include */
 
@@ -36,25 +34,29 @@
 #define NOTE_PREFIX "Note: "
 #define ERROR_PREFIX "curl: "
 
-static void voutf(struct GlobalConfig *config,
-                  const char *prefix,
+static void voutf(const char *prefix,
+                  const char *fmt,
+                  va_list ap) CURL_PRINTF(2, 0);
+
+static void voutf(const char *prefix,
                   const char *fmt,
                   va_list ap)
 {
-  size_t width = (79 - strlen(prefix));
-  if(!config->silent) {
+  size_t width = (get_terminal_columns() - strlen(prefix));
+  DEBUGASSERT(!strchr(fmt, '\n'));
+  if(!global->silent) {
     size_t len;
     char *ptr;
     char *print_buffer;
 
-    print_buffer = curlx_mvaprintf(fmt, ap);
+    print_buffer = vaprintf(fmt, ap);
     if(!print_buffer)
       return;
     len = strlen(print_buffer);
 
     ptr = print_buffer;
     while(len > 0) {
-      fputs(prefix, config->errors);
+      fputs(prefix, tool_stderr);
 
       if(len > width) {
         size_t cut = width-1;
@@ -62,18 +64,19 @@ static void voutf(struct GlobalConfig *config,
         while(!ISBLANK(ptr[cut]) && cut) {
           cut--;
         }
-        if(0 == cut)
+        if(cut == 0)
           /* not a single cutting position was found, just cut it at the
              max text width then! */
           cut = width-1;
 
-        (void)fwrite(ptr, cut + 1, 1, config->errors);
-        fputs("\n", config->errors);
+        (void)fwrite(ptr, cut + 1, 1, tool_stderr);
+        fputs("\n", tool_stderr);
         ptr += cut + 1; /* skip the space too */
         len -= cut + 1;
       }
       else {
-        fputs(ptr, config->errors);
+        fputs(ptr, tool_stderr);
+        fputs("\n", tool_stderr);
         len = 0;
       }
     }
@@ -85,12 +88,12 @@ static void voutf(struct GlobalConfig *config,
  * Emit 'note' formatted message on configured 'errors' stream, if verbose was
  * selected.
  */
-void notef(struct GlobalConfig *config, const char *fmt, ...)
+void notef(const char *fmt, ...)
 {
   va_list ap;
   va_start(ap, fmt);
-  if(config->tracetype)
-    voutf(config, NOTE_PREFIX, fmt, ap);
+  if(global->tracetype)
+    voutf(NOTE_PREFIX, fmt, ap);
   va_end(ap);
 }
 
@@ -98,28 +101,30 @@ void notef(struct GlobalConfig *config, const char *fmt, ...)
  * Emit warning formatted message on configured 'errors' stream unless
  * mute (--silent) was selected.
  */
-
-void warnf(struct GlobalConfig *config, const char *fmt, ...)
+void warnf(const char *fmt, ...)
 {
   va_list ap;
   va_start(ap, fmt);
-  voutf(config, WARN_PREFIX, fmt, ap);
+  voutf(WARN_PREFIX, fmt, ap);
   va_end(ap);
 }
+
 /*
  * Emit help formatted message on given stream. This is for errors with or
  * related to command line arguments.
  */
-void helpf(FILE *errors, const char *fmt, ...)
+void helpf(const char *fmt, ...)
 {
   if(fmt) {
     va_list ap;
     va_start(ap, fmt);
-    fputs("curl: ", errors); /* prefix it */
-    vfprintf(errors, fmt, ap);
+    DEBUGASSERT(!strchr(fmt, '\n'));
+    fputs("curl: ", tool_stderr); /* prefix it */
+    vfprintf(tool_stderr, fmt, ap);
     va_end(ap);
+    fputs("\n", tool_stderr); /* newline it */
   }
-  fprintf(errors, "curl: try 'curl --help' "
+  fprintf(tool_stderr, "curl: try 'curl --help' "
 #ifdef USE_MANUAL
           "or 'curl --manual' "
 #endif
@@ -130,12 +135,12 @@ void helpf(FILE *errors, const char *fmt, ...)
  * Emit error message on error stream if not muted. When errors are not tied
  * to command line arguments, use helpf() for such errors.
  */
-void errorf(struct GlobalConfig *config, const char *fmt, ...)
+void errorf(const char *fmt, ...)
 {
-  if(!config->silent) {
+  if(!global->silent || global->showerror) {
     va_list ap;
     va_start(ap, fmt);
-    voutf(config, ERROR_PREFIX, fmt, ap);
+    voutf(ERROR_PREFIX, fmt, ap);
     va_end(ap);
   }
 }
