@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,18 +18,10 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "tool_setup.h"
-
-#ifdef HAVE_FSETXATTR
-#  include <sys/xattr.h> /* header from libc, not from libattr */
-#  define USE_XATTR
-#elif defined(__FreeBSD_version) && (__FreeBSD_version > 500000)
-#  include <sys/types.h>
-#  include <sys/extattr.h>
-#  define USE_XATTR
-#endif
-
 #include "tool_xattr.h"
 
 #include "memdebug.h" /* keep this as LAST include */
@@ -44,26 +36,21 @@ static const struct xattr_mapping {
   /* mappings proposed by
    * https://freedesktop.org/wiki/CommonExtendedAttributes/
    */
-  { "user.xdg.origin.url", CURLINFO_EFFECTIVE_URL },
-  { "user.mime_type",      CURLINFO_CONTENT_TYPE },
-  { NULL,                  CURLINFO_NONE } /* last element, abort loop here */
+  { "user.xdg.referrer.url", CURLINFO_REFERER },
+  { "user.mime_type",        CURLINFO_CONTENT_TYPE },
+  { NULL,                    CURLINFO_NONE } /* last element, abort here */
 };
 
-/* returns TRUE if a new URL is returned, that then needs to be freed */
+/* returns a new URL that needs to be freed */
 /* @unittest: 1621 */
-#ifdef UNITTESTS
-bool stripcredentials(char **url);
-#else
-static
-#endif
-bool stripcredentials(char **url)
+UNITTEST char *stripcredentials(const char *url)
 {
   CURLU *u;
   CURLUcode uc;
   char *nurl;
   u = curl_url();
   if(u) {
-    uc = curl_url_set(u, CURLUPART_URL, *url, 0);
+    uc = curl_url_set(u, CURLUPART_URL, url, CURLU_GUESS_SCHEME);
     if(uc)
       goto error;
 
@@ -81,59 +68,64 @@ bool stripcredentials(char **url)
 
     curl_url_cleanup(u);
 
-    *url = nurl;
-    return TRUE;
+    return nurl;
   }
-  error:
+error:
   curl_url_cleanup(u);
-  return FALSE;
+  return NULL;
 }
 
+static int xattr(int fd,
+                 const char *attr, /* name of the xattr */
+                 const char *value)
+{
+  int err = 0;
+  if(value) {
+#ifdef DEBUGBUILD
+    if(getenv("CURL_FAKE_XATTR")) {
+      printf("%s => %s\n", attr, value);
+      return 0;
+    }
+#endif
+#ifdef HAVE_FSETXATTR_6
+    err = fsetxattr(fd, attr, value, strlen(value), 0, 0);
+#elif defined(HAVE_FSETXATTR_5)
+    err = fsetxattr(fd, attr, value, strlen(value), 0);
+#elif defined(__FreeBSD_version) || defined(__MidnightBSD_version)
+    {
+      ssize_t rc = extattr_set_fd(fd, EXTATTR_NAMESPACE_USER,
+                                  attr, value, strlen(value));
+      /* FreeBSD's extattr_set_fd returns the length of the extended
+         attribute */
+      err = (rc < 0 ? -1 : 0);
+    }
+#endif
+  }
+  return err;
+}
 /* store metadata from the curl request alongside the downloaded
  * file using extended attributes
  */
-int fwrite_xattr(CURL *curl, int fd)
+int fwrite_xattr(CURL *curl, const char *url, int fd)
 {
   int i = 0;
-  int err = 0;
+  int err = xattr(fd, "user.creator", "curl");
 
   /* loop through all xattr-curlinfo pairs and abort on a set error */
-  while(err == 0 && mappings[i].attr != NULL) {
+  while(!err && mappings[i].attr) {
     char *value = NULL;
     CURLcode result = curl_easy_getinfo(curl, mappings[i].info, &value);
-    if(!result && value) {
-      bool freeptr = FALSE;
-      if(CURLINFO_EFFECTIVE_URL == mappings[i].info)
-        freeptr = stripcredentials(&value);
-      if(value) {
-#ifdef HAVE_FSETXATTR_6
-        err = fsetxattr(fd, mappings[i].attr, value, strlen(value), 0, 0);
-#elif defined(HAVE_FSETXATTR_5)
-        err = fsetxattr(fd, mappings[i].attr, value, strlen(value), 0);
-#elif defined(__FreeBSD_version)
-        {
-          ssize_t rc = extattr_set_fd(fd, EXTATTR_NAMESPACE_USER,
-                                      mappings[i].attr, value, strlen(value));
-          /* FreeBSD's extattr_set_fd returns the length of the extended
-             attribute */
-          err = (rc < 0 ? -1 : 0);
-        }
-#endif
-        if(freeptr)
-          curl_free(value);
-      }
-    }
+    if(!result && value)
+      err = xattr(fd, mappings[i].attr, value);
     i++;
   }
-
+  if(!err) {
+    char *nurl = stripcredentials(url);
+    if(!nurl)
+      return 1;
+    err = xattr(fd, "user.xdg.origin.url", nurl);
+    curl_free(nurl);
+  }
   return err;
 }
-#else
-int fwrite_xattr(CURL *curl, int fd)
-{
-  (void)curl;
-  (void)fd;
-
-  return 0;
-}
 #endif
