@@ -21,36 +21,36 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
+#include "first.h"
 
 #include "memdebug.h"
 
-#define WITH_PROXY     "http://usingproxy.com/"
-#define WITHOUT_PROXY  libtest_arg2
-
 static void proxystat(CURL *curl)
 {
   long wasproxy;
   if(!curl_easy_getinfo(curl, CURLINFO_USED_PROXY, &wasproxy)) {
-    printf("This %sthe proxy\n", wasproxy ? "used ":
-           "DID NOT use ");
+    curl_mprintf("This %sthe proxy\n", wasproxy ? "used ":
+                 "DID NOT use ");
   }
 }
 
-CURLcode test(char *URL)
+static CURLcode test_lib536(const char *URL)
 {
   CURLcode res = CURLE_OK;
   CURL *curl;
   struct curl_slist *host = NULL;
 
+  static const char *url_with_proxy = "http://usingproxy.com/";
+  const char *url_without_proxy = libtest_arg2;
+
   if(curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
-    fprintf(stderr, "curl_global_init() failed\n");
+    curl_mfprintf(stderr, "curl_global_init() failed\n");
     return TEST_ERR_MAJOR_BAD;
   }
 
   curl = curl_easy_init();
   if(!curl) {
-    fprintf(stderr, "curl_easy_init() failed\n");
+    curl_mfprintf(stderr, "curl_easy_init() failed\n");
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
   }
@@ -61,14 +61,14 @@ CURLcode test(char *URL)
 
   test_setopt(curl, CURLOPT_RESOLVE, host);
   test_setopt(curl, CURLOPT_PROXY, URL);
-  test_setopt(curl, CURLOPT_URL, WITH_PROXY);
+  test_setopt(curl, CURLOPT_URL, url_with_proxy);
   test_setopt(curl, CURLOPT_NOPROXY, "goingdirect.com");
   test_setopt(curl, CURLOPT_VERBOSE, 1L);
 
   res = curl_easy_perform(curl);
   if(!res) {
     proxystat(curl);
-    test_setopt(curl, CURLOPT_URL, WITHOUT_PROXY);
+    test_setopt(curl, CURLOPT_URL, url_without_proxy);
     res = curl_easy_perform(curl);
     if(!res)
       proxystat(curl);
